@@ -1,19 +1,17 @@
 # -*- coding: utf-8 -*-
-"""Field classes for formatting and validating the serialized object.
+"""Field classes for various types of data.
 """
 from __future__ import absolute_import
 
-from decimal import Decimal as MyDecimal, ROUND_HALF_EVEN
-from functools import partial
 import datetime as dt
-import inspect
 import uuid
 import warnings
+from decimal import Decimal
+from functools import partial
 
 from marshmallow import validate, utils, class_registry
 from marshmallow.base import FieldABC, SchemaABC
-from marshmallow.compat import (text_type, OrderedDict, iteritems, total_seconds,
-                                basestring)
+from marshmallow.compat import text_type, iteritems, total_seconds, basestring
 from marshmallow.exceptions import (
     MarshallingError,
     UnmarshallingError,
@@ -22,6 +20,7 @@ from marshmallow.exceptions import (
     ValidationError,
 )
 
+
 __all__ = [
     'Marshaller',
     'Unmarshaller',
@@ -43,7 +42,6 @@ __all__ = [
     'Date',
     'TimeDelta',
     'Fixed',
-    'ZERO',
     'Price',
     'Url',
     'URL',
@@ -55,9 +53,11 @@ __all__ = [
     'Str',
     'Bool',
     'Int',
+    'null',
+    'missing',
 ]
 
-class _null(object):
+class _Null(object):
 
     def __bool__(self):
         return False
@@ -67,7 +67,7 @@ class _null(object):
     def __repr__(self):
         return '<marshmallow.fields.null>'
 
-class _missing(_null):
+class _Missing(_Null):
 
     def __repr__(self):
         return '<marshmallow.fields.missing>'
@@ -76,12 +76,12 @@ class _missing(_null):
 # fields so that `Field._call_with_validation` is invoked, even when the
 # object to serialize has the nested attribute set to None. Therefore,
 # `RegistryErrors` are properly raised.
-null = _null()
+null = _Null()
 
 # Singleton value that indicates that a field's value is missing from input
 # dict passed to :meth:`Schema.load`. If the field's value is not required,
 # it's ``default`` value is used.
-missing = _missing()
+missing = _Missing()
 
 def _call_and_store(getter_func, data, field_name, field_obj, errors_dict,
                exception_class, strict=False):
@@ -140,30 +140,45 @@ class Marshaller(object):
         self.prefix = prefix
         #: Dictionary of errors stored during serialization
         self.errors = {}
+        #: True while serializing a collection
+        self.__pending = False
 
-    def serialize(self, obj, fields_dict, many=False, strict=False):
+    def serialize(self, obj, fields_dict, many=False, strict=False, skip_missing=False,
+                  accessor=None, dict_class=None):
         """Takes raw data (a dict, list, or other object) and a dict of
         fields to output and serializes the data based on those fields.
 
         :param obj: The actual object(s) from which the fields are taken from
         :param dict fields_dict: Mapping of field names to :class:`Field` objects.
-        :param bool many: Set to ``True`` if ``data`` should be serialized as
+        :param bool many: Set to `True` if ``data`` should be serialized as
             a collection.
-        :param bool strict: If ``True``, raise errors if invalid data are passed in
+        :param bool strict: If `True`, raise errors if invalid data are passed in
             instead of failing silently and storing the errors.
-        :return: An OrderedDict of the marshalled data
+        :param skip_missing: If `True`, skip key:value pairs when ``value`` is `None`.
+        :param callable accessor: Function to use for getting values from ``obj``.
+        :param type dict_class: Dictionary class used to construct the output.
+        :return: A dictionary of the marshalled data
 
         .. versionchanged:: 1.0.0
             Renamed from ``marshal``.
         """
+        dict_class = dict_class or dict
+        # Reset errors dict if not serializing a collection
+        if not self.__pending:
+            self.errors = {}
         if many and obj is not None:
-            return [self.serialize(d, fields_dict, many=False, strict=strict)
+            self.__pending = True
+            ret = [self.serialize(d, fields_dict, many=False, strict=strict,
+                                    dict_class=dict_class)
                     for d in obj]
+            self.__pending = False
+            return ret
         items = []
         for attr_name, field_obj in iteritems(fields_dict):
             key = ''.join([self.prefix, attr_name])
+            getter = lambda d: field_obj.serialize(attr_name, d, accessor=accessor)
             value = _call_and_store(
-                getter_func=partial(field_obj.serialize, attr_name),
+                getter_func=getter,
                 data=obj,
                 field_name=key,
                 field_obj=field_obj,
@@ -171,8 +186,10 @@ class Marshaller(object):
                 exception_class=MarshallingError,
                 strict=strict
             )
+            if (value is missing) or (value is None and skip_missing):
+                continue
             items.append((key, value))
-        return OrderedDict(items)
+        return dict_class(items)
 
     # Make an instance callable
     __call__ = serialize
@@ -181,17 +198,16 @@ class Marshaller(object):
 class Unmarshaller(object):
     """Callable class responsible for deserializing data and storing errors.
 
-    :param str prefix: Optional prefix that will be prepended to all the
-        serialized field names.
-
     .. versionadded:: 1.0.0
     """
-    def __init__(self, prefix=''):
+    def __init__(self):
         #: Dictionary of errors stored during deserialization
         self.errors = {}
+        #: True while deserializing a collection
+        self.__pending = False
 
     def _validate(self, validators, output, strict=False):
-        """Perform schema-level validation. Stores errors if ``strict`` is ``False``.
+        """Perform schema-level validation. Stores errors if ``strict`` is `False`.
         """
         for validator_func in validators:
             try:
@@ -209,26 +225,34 @@ class Unmarshaller(object):
         return output
 
     def deserialize(self, data, fields_dict, many=False, validators=None,
-            preprocess=None, postprocess=None, strict=False):
+            preprocess=None, postprocess=None, strict=False, dict_class=None):
         """Deserialize ``data`` based on the schema defined by ``fields_dict``.
 
         :param dict data: The data to deserialize.
         :param dict fields_dict: Mapping of field names to :class:`Field` objects.
-        :param bool many: Set to ``True`` if ``data`` should be deserialized as
+        :param bool many: Set to `True` if ``data`` should be deserialized as
             a collection.
         :param list validators: List of validation functions to apply to the
             deserialized dictionary.
         :param list preprocess: List of pre-processing functions.
         :param list postprocess: List of post-processing functions.
-        :param bool strict: If ``True``, raise errors if invalid data are passed in
+        :param bool strict: If `True`, raise errors if invalid data are passed in
             instead of failing silently and storing the errors.
-        :return: An OrderedDict of the deserialized data.
+        :param type dict_class: Dictionary class used to construct the output.
+        :return: A dictionary of the deserialized data.
         """
+        dict_class = dict_class or dict
+        # Reset errors if not deserializing a collection
+        if not self.__pending:
+            self.errors = {}
         if many and data is not None:
-            return [self.deserialize(d, fields_dict, many=False,
+            self.__pending = True
+            ret = [self.deserialize(d, fields_dict, many=False,
                         validators=validators, preprocess=preprocess,
-                        postprocess=postprocess, strict=strict)
+                        postprocess=postprocess, strict=strict, dict_class=dict_class)
                     for d in data]
+            self.__pending = False
+            return ret
         items = []
         for attr_name, field_obj in iteritems(fields_dict):
             if attr_name not in fields_dict:
@@ -248,7 +272,7 @@ class Unmarshaller(object):
             )
             if raw_value is not missing:
                 items.append((key, value))
-        ret = OrderedDict(items)
+        ret = dict_class(items)
 
         if preprocess:
             preprocess = preprocess or []
@@ -273,17 +297,20 @@ class Field(FieldABC):
     data does not need to be formatted before being serialized or deserialized.
 
     :param default: Default serialization value for the field if the attribute is
-        ``None``.
+        `None`. May be a value or a callable.
     :param str attribute: The name of the attribute to get the value from. If
-        ``None``, assumes the attribute has the same name as the field.
+        `None`, assumes the attribute has the same name as the field.
     :param str error: Error message stored upon validation failure.
     :param callable validate: Validator or collection of validators that are called
         during deserialization. Validator takes a field's input value as
         its only parameter and returns a boolean.
-        If it returns ``False``, an :exc:`UnmarshallingError` is raised.
+        If it returns `False`, an :exc:`UnmarshallingError` is raised.
     :param bool required: Raise an :exc:`UnmarshallingError` if the field value
         is not supplied during deserialization.
     :param metadata: Extra arguments to be stored as metadata.
+
+    .. versionchanged:: 1.0.0
+        Deprecated `error` parameter. Raise a :exc:`marshmallow.ValidationError` instead.
     """
     # Some fields, such as Method fields and Function fields, are not expected
     #  to exists as attributes on the objects to serialize. Set this to False
@@ -295,7 +322,12 @@ class Field(FieldABC):
                  validate=None, required=False, **metadata):
         self.default = default
         self.attribute = attribute
+        if error:
+            warnings.warn('The error parameter is deprecated. Raise a '
+                          'marshmallow.ValidationError in your validators '
+                          'instead.', category=DeprecationWarning)
         self.error = error
+        self.validate = validate
         if utils.is_iterable_but_not_string(validate):
             if not utils.is_generator(validate):
                 self.validators = validate
@@ -314,13 +346,20 @@ class Field(FieldABC):
         Field._creation_index += 1
         self.parent = FieldABC.parent
 
-    def get_value(self, attr, obj):
+    def __repr__(self):
+        return ('<fields.{ClassName}(default={self.default!r}, '
+                'attribute={self.attribute!r}, error={self.error!r}, '
+                'validate={self.validate}, required={self.required})>'
+                .format(ClassName=self.__class__.__name__, self=self))
+
+    def get_value(self, attr, obj, accessor=None):
         """Return the value for a given key from an object."""
         # NOTE: Use getattr instead of direct attribute access here so that
         # subclasses aren't required to define `attribute` member
         attribute = getattr(self, 'attribute', None)
+        accessor_func = accessor or utils.get_value
         check_key = attr if attribute is None else attribute
-        return utils.get_value(check_key, obj)
+        return accessor_func(check_key, obj)
 
     def _validate(self, value):
         """Perform validation on ``value``. Raise a :exc:`ValidationError` if validation
@@ -361,22 +400,26 @@ class Field(FieldABC):
         except ValidationError as err:
             raise exception_class(err)
         # Reraise errors, wrapping with exception_class
-        except Exception as error:
-            raise exception_class(getattr(self, 'error', None) or error)
+        # except Exception as error:
+        #     raise exception_class(getattr(self, 'error', None) or error)
 
-    def serialize(self, attr, obj):
+    def serialize(self, attr, obj, accessor=None):
         """Pulls the value for the given key from the object, applies the
         field's formatting and returns the result.
 
         :param str attr: The attibute or key to get from the object.
         :param str obj: The object to pull the key from.
+        :param callable accessor: Function used to pull values from ``obj``.
         :raise MarshallingError: In case of formatting problem
         """
-        value = self.get_value(attr, obj)
+        value = self.get_value(attr, obj, accessor=accessor)
         if value is None and self._CHECK_ATTRIBUTE:
             if hasattr(self, 'default') and self.default != null:
-                return self.default
-        func = partial(self._serialize, value, attr, obj)
+                if callable(self.default):
+                    return self.default()
+                else:
+                    return self.default
+        func = lambda: self._serialize(value, attr, obj)
         return self._call_and_reraise(func, MarshallingError)
 
     def deserialize(self, value):
@@ -448,7 +491,7 @@ class Nested(Field):
         to nest, or ``"self"`` to nest the :class:`Schema` within itself.
     :param default: Default value to if attribute is missing or None
     :param tuple exclude: A list or tuple of fields to exclude.
-    :param only: A tuple or string of the field(s) to marshal. If ``None``, all fields
+    :param only: A tuple or string of the field(s) to marshal. If `None`, all fields
         will be marshalled. If a field name (string) is given, only a single
         value will be returned as output instead of a dictionary.
         This parameter takes precedence over ``exclude``.
@@ -465,30 +508,10 @@ class Nested(Field):
         self.only = only
         self.exclude = exclude
         self.many = many
-        self.__schema = None
-        self.__updated_fields = False  # ensures serializer fields are updated only once
+        self.__schema = None  # Cached Schema instance
+        self.__updated_fields = False
         super(Nested, self).__init__(default=default, **kwargs)
 
-    def __get_fields_to_marshal(self, all_fields):
-        """Filter all_fields based on self.only and self.exclude """
-        # Default 'only' to all the nested fields
-        ret = OrderedDict()
-        if all_fields is None:
-            return ret
-        elif isinstance(self.only, basestring):
-            ret[self.only] = all_fields[self.only]
-            return ret
-        else:
-            only = set(all_fields) if self.only is None else set(self.only)
-        if self.exclude and self.only:
-            # Make sure that only takes precedence
-            exclude = set(self.exclude) - only
-        else:
-            exclude = set([]) if self.exclude is None else set(self.exclude)
-        filtered = ((k, v) for k, v in iteritems(all_fields)
-                    if k in only and k not in exclude)
-        return OrderedDict(filtered)
-
     @property
     def schema(self):
         """The nested Schema object.
@@ -496,24 +519,32 @@ class Nested(Field):
         .. versionchanged:: 1.0.0
             Renamed from `serializer` to `schema`
         """
-        # Cache the serializer instance
+        # Ensure that only parameter is a tuple
+        if isinstance(self.only, basestring):
+            only = (self.only, )
+        else:
+            only = self.only
         if not self.__schema:
             if isinstance(self.nested, SchemaABC):
                 self.__schema = self.nested
             elif isinstance(self.nested, type) and \
                     issubclass(self.nested, SchemaABC):
-                self.__schema = self.nested(None, many=self.many)
+                self.__schema = self.nested(many=self.many,
+                        only=only, exclude=self.exclude)
             elif isinstance(self.nested, basestring):
                 if self.nested == 'self':
                     parent_class = self.parent.__class__
-                    self.__schema = parent_class(many=self.many)
+                    self.__schema = parent_class(many=self.many, only=only,
+                            exclude=self.exclude)
                 else:
                     schema_class = class_registry.get_class(self.nested)
-                    self.__schema = schema_class(None, many=self.many)
+                    self.__schema = schema_class(many=self.many,
+                            only=only, exclude=self.exclude)
             else:
                 raise ForcedError(ValueError('Nested fields must be passed a '
                                     'Schema, not {0}.'
                                     .format(self.nested.__class__)))
+        self.__schema.ordered = getattr(self.parent, 'ordered', False)
         # Inherit context from parent
         self.__schema.context.update(getattr(self.parent, 'context', {}))
         return self.__schema
@@ -521,17 +552,12 @@ class Nested(Field):
     def _serialize(self, nested_obj, attr, obj):
         if self.allow_null and nested_obj is None:
             return None
-        self.schema.many = self.many
-        self.schema.obj = nested_obj
         if not self.__updated_fields:
-            self.__updated_fields = True
             self.schema._update_fields(nested_obj)
-        fields = self.__get_fields_to_marshal(self.schema.fields)
+            self.__updated_fields = True
         try:
-            # We call the protected _marshal method instead of _dump
-            # because we need to pass the this field's ``many`` attribute as
-            # an argument, which dump would not allow
-            ret = self.schema._marshal(nested_obj, fields, many=self.many)
+            ret = self.schema.dump(nested_obj, many=self.many,
+                    update_fields=not self.__updated_fields).data
         except TypeError as err:
             raise TypeError('Could not marshal nested object due to error:\n"{0}"\n'
                             'If the nested object is a collection, you need to set '
@@ -559,8 +585,8 @@ class List(Field):
 
     :param Field cls_or_instance: A field class or instance.
     :param bool default: Default value for serialization.
-    :param bool allow_none: If ``True``, ``None`` will be serialized to ``None``.
-        If ``False``, ``None`` will serialize to an empty list.
+    :param bool allow_none: If `True`, `None` will be serialized to `None`.
+        If `False`, `None` will serialize to an empty list.
     :param kwargs: The same keyword arguments that :class:`Field` receives.
     """
     def __init__(self, cls_or_instance, default=None, allow_none=False, **kwargs):
@@ -569,13 +595,13 @@ class List(Field):
             self.default = []
         if isinstance(cls_or_instance, type):
             if not issubclass(cls_or_instance, FieldABC):
-                raise MarshallingError('The type of the list elements '
+                raise ValueError('The type of the list elements '
                                            'must be a subclass of '
                                            'marshmallow.base.FieldABC')
             self.container = cls_or_instance()
         else:
             if not isinstance(cls_or_instance, FieldABC):
-                raise MarshallingError('The instances of the list '
+                raise ValueError('The instances of the list '
                                            'elements must be of type '
                                            'marshmallow.base.FieldABC')
             self.container = cls_or_instance
@@ -652,8 +678,8 @@ class Number(Field):
             if value is None:
                 return self.default
             return self._format_num(value)
-        except ValueError as ve:
-            raise exception_class(ve)
+        except (TypeError, ValueError) as err:
+            raise exception_class(getattr(self, 'error', None) or err)
 
     def _serialize(self, value, attr, obj):
         return self._validated(value, MarshallingError)
@@ -673,7 +699,7 @@ class Integer(Number):
 
     def __init__(self, default=0, attribute=None, as_string=False, error=None, **kwargs):
         self.as_string = as_string
-        super(Number, self).__init__(default=default, attribute=attribute,
+        super(Integer, self).__init__(default=default, attribute=attribute,
             error=error, **kwargs)
 
 class Boolean(Field):
@@ -682,10 +708,10 @@ class Boolean(Field):
     :param kwargs: The same keyword arguments that :class:`Field` receives.
     """
 
-    #: Values that will deserialize to ``True``. If an empty set, any non-falsy
-    #  value will deserialize to ``True``.
+    #: Values that will deserialize to `True`. If an empty set, any non-falsy
+    #  value will deserialize to `True`.
     truthy = set()
-    #: Values that will deserialize to ``False``.
+    #: Values that will deserialize to `False`.
     falsy = set(['False', 'false', '0', 'null', 'None'])
 
     def _serialize(self, value, attr, obj):
@@ -716,11 +742,11 @@ class FormattedString(Field):
     from the python stdlib.
     ::
 
-        class UserSer(Schema):
+        class UserSchema(Schema):
             name = fields.String()
             greeting = fields.FormattedString('Hello {name}')
 
-        ser = UserSer()
+        ser = UserSchema()
         res = ser.dump(user)
         res.data  # => {'name': 'Monty', 'greeting': 'Hello Monty'}
     """
@@ -733,7 +759,7 @@ class FormattedString(Field):
             data = utils.to_marshallable_type(obj)
             return self.src_str.format(**data)
         except (TypeError, IndexError) as error:
-            raise MarshallingError(error)
+            raise MarshallingError(getattr(self, 'error', None) or error)
 
 
 class Float(Number):
@@ -795,10 +821,10 @@ class DateTime(Field):
         ex. ``"Sun, 10 Nov 2013 07:23:45 -0000"``
 
     :param str format: Either ``"rfc"`` (for RFC822), ``"iso"`` (for ISO8601),
-        or a date format string. If ``None``, defaults to "rfc".
+        or a date format string. If `None`, defaults to "rfc".
     :param default: Default value for the field if the attribute is not set.
     :param str attribute: The name of the attribute to get the value from. If
-        ``None``, assumes the attribute has the same name as the field.
+        `None`, assumes the attribute has the same name as the field.
     :param kwargs: The same keyword arguments that :class:`Field` receives.
 
     """
@@ -818,7 +844,10 @@ class DateTime(Field):
             self.dateformat = self.dateformat or self.DEFAULT_FORMAT
             format_func = DATEFORMAT_SERIALIZATION_FUNCS.get(self.dateformat, None)
             if format_func:
-                return format_func(value, localtime=self.localtime)
+                try:
+                    return format_func(value, localtime=self.localtime)
+                except (AttributeError, ValueError) as err:
+                    raise MarshallingError(getattr(self, 'error', None) or err)
             else:
                 return value.strftime(self.dateformat)
 
@@ -833,6 +862,8 @@ class DateTime(Field):
                 return func(value)
             except TypeError:
                 raise err
+            except (AttributeError, ValueError) as err:
+                raise UnmarshallingError(getattr(self, 'error', None) or err)
         elif utils.dateutil_available:
             try:
                 return utils.from_datestring(value)
@@ -864,8 +895,8 @@ class Time(Field):
         try:
             ret = value.isoformat()
         except AttributeError:
-            raise MarshallingError('{0} cannot be formatted as a time.'
-                                    .format(repr(value)))
+            msg = '{0!r} cannot be formatted as a time.'.format(value)
+            raise MarshallingError(getattr(self, 'error', None) or msg)
         if value.microsecond:
             return ret[:12]
         return ret
@@ -875,9 +906,8 @@ class Time(Field):
         try:
             return utils.from_iso_time(value)
         except TypeError:
-            raise UnmarshallingError(
-                'Could not deserialize {0!r} to a time object.'.format(value)
-            )
+            msg = 'Could not deserialize {0!r} to a time object.'.format(value)
+            raise UnmarshallingError(getattr(self, 'error', None) or msg)
 
 class Date(Field):
     """ISO8601-formatted date string.
@@ -889,8 +919,8 @@ class Date(Field):
         try:
             return value.isoformat()
         except AttributeError:
-            raise MarshallingError('{0} cannot be formatted as a date.'
-                                    .format(repr(value)))
+            msg = '{0} cannot be formatted as a date.'.format(repr(value))
+            raise MarshallingError(getattr(self, 'error', None) or msg)
         return value
 
     def _deserialize(self, value):
@@ -900,9 +930,8 @@ class Date(Field):
         try:
             return utils.from_iso_date(value)
         except TypeError:
-            raise UnmarshallingError(
-                'Could not deserialize {0!r} to a date object.'.format(value)
-            )
+            msg = 'Could not deserialize {0!r} to a date object.'.format(value)
+            raise UnmarshallingError(getattr(self, 'error', None) or msg)
 
 
 class TimeDelta(Field):
@@ -916,22 +945,20 @@ class TimeDelta(Field):
         try:
             return total_seconds(value)
         except AttributeError:
-            raise MarshallingError('{0} cannot be formatted as a timedelta.'
-                                    .format(repr(value)))
+            msg = '{0} cannot be formatted as a timedelta.'.format(repr(value))
+            raise MarshallingError(getattr(self, 'error', None) or msg)
         return value
 
     def _deserialize(self, value):
         """Deserialize a value in seconds to a :class:`datetime.timedelta`
         object.
         """
-        return dt.timedelta(seconds=float(value))
+        try:
+            return dt.timedelta(seconds=float(value))
+        except (AttributeError, ValueError) as err:
+            raise UnmarshallingError(getattr(self, 'error', None) or err)
 
 
-ZERO = MyDecimal()
-
-def _to_fixed(value, precision):
-    return text_type(value.quantize(precision, rounding=ROUND_HALF_EVEN))
-
 class Fixed(Number):
     """A fixed-precision number as a string.
 
@@ -942,7 +969,7 @@ class Fixed(Number):
                  *args, **kwargs):
         super(Fixed, self).__init__(default=default, attribute=attribute, error=error,
                             *args, **kwargs)
-        self.precision = MyDecimal('0.' + '0' * (decimals - 1) + '1')
+        self.precision = Decimal('0.' + '0' * (decimals - 1) + '1')
 
     def _serialize(self, value, attr, obj):
         return self._validated(value, MarshallingError)
@@ -956,10 +983,12 @@ class Fixed(Number):
         try:
             dvalue = utils.float_to_decimal(float(value))
         except (TypeError, ValueError) as err:
-            raise exception_class(err)
-        if not dvalue.is_normal() and dvalue != ZERO:
-            raise exception_class('Invalid Fixed precision number.')
-        return _to_fixed(dvalue, self.precision)
+            raise exception_class(getattr(self, 'error', None) or err)
+        if not dvalue.is_normal() and dvalue != utils.ZERO_DECIMAL:
+            raise exception_class(
+                getattr(self, 'error', None) or 'Invalid Fixed precision number.'
+            )
+        return utils.decimal_to_fixed(dvalue, self.precision)
 
 
 class Price(Fixed):
@@ -970,13 +999,22 @@ class Price(Fixed):
     def __init__(self, decimals=2, default='0.00', **kwargs):
         super(Price, self).__init__(decimals=decimals, default=default, **kwargs)
 
+class ValidatedField(Field):
+    """A field that is validates input on serialization."""
 
-class Url(Field):
+    def _validated(self, value):
+        raise NotImplementedError('Must implement _validate method')
+
+    def _serialize(self, value, *args, **kwargs):
+        ret = super(ValidatedField, self)._serialize(value, *args, **kwargs)
+        return self._validated(ret)
+
+class Url(ValidatedField):
     """A validated URL field.
 
     :param default: Default value for the field if the attribute is not set.
     :param str attribute: The name of the attribute to get the value from. If
-        ``None``, assumes the attribute has the same name as the field.
+        `None`, assumes the attribute has the same name as the field.
     :param bool relative: Allow relative URLs.
     :param kwargs: The same keyword arguments that :class:`Field` receives.
     """
@@ -985,32 +1023,29 @@ class Url(Field):
         super(Url, self).__init__(default=default, attribute=attribute,
                 *args, **kwargs)
         self.relative = relative
+        # Insert validation into self.validators so that multiple errors can be
+        # stored.
         self.validators.insert(0, partial(validate.url,
             relative=self.relative, error=getattr(self, 'error')))
 
+    def _validated(self, value):
+        return validate.url(value, relative=self.relative, error=getattr(self, 'error'))
+
 URL = Url
 
-class Email(Field):
+class Email(ValidatedField):
     """A validated email field.
 
     :param kwargs: The same keyword arguments that :class:`Field` receives.
     """
     def __init__(self, *args, **kwargs):
         super(Email, self).__init__(*args, **kwargs)
+        # Insert validation into self.validators so that multiple errors can be
+        # stored.
         self.validators.insert(0, partial(validate.email, error=getattr(self, 'error')))
 
-
-def _get_args(func):
-    """Return a tuple of argument names for a function."""
-    return inspect.getargspec(func).args
-
-
-def _callable(obj):
-    """Check that an object is callable, else raise a :exc:`ValueError`.
-    """
-    if not callable(obj):
-        raise ValueError('Object {0!r} is not callable.'.format(obj))
-    return obj
+    def _validated(self, value):
+        return validate.email(value, error=getattr(self, 'error'))
 
 
 class Method(Field):
@@ -1037,8 +1072,8 @@ class Method(Field):
 
     def _serialize(self, value, attr, obj):
         try:
-            method = _callable(getattr(self.parent, self.method_name, None))
-            if len(_get_args(method)) > 2:
+            method = utils.callable_or_raise(getattr(self.parent, self.method_name, None))
+            if len(utils.get_func_args(method)) > 2:
                 if self.parent.context is None:
                     msg = 'No context available for Method field {0!r}'.format(attr)
                     raise MarshallingError(msg)
@@ -1051,7 +1086,7 @@ class Method(Field):
     def _deserialize(self, value):
         if self.deserialize_method_name:
             try:
-                method = _callable(
+                method = utils.callable_or_raise(
                     getattr(self.parent, self.deserialize_method_name, None)
                 )
                 return method(value)
@@ -1074,15 +1109,15 @@ class Function(Field):
 
     def __init__(self, func, deserialize=None, **kwargs):
         super(Function, self).__init__(**kwargs)
-        self.func = _callable(func)
+        self.func = utils.callable_or_raise(func)
         if deserialize:
-            self.deserialize_func = _callable(deserialize)
+            self.deserialize_func = utils.callable_or_raise(deserialize)
         else:
             self.deserialize_func = None
 
     def _serialize(self, value, attr, obj):
         try:
-            if len(_get_args(self.func)) > 1:
+            if len(utils.get_func_args(self.func)) > 1:
                 if self.parent.context is None:
                     msg = 'No context available for Function field {0!r}'.format(attr)
                     raise MarshallingError(msg)
@@ -1107,7 +1142,7 @@ class Select(Field):
     :param choices: A list of valid values.
     :param default: Default value for the field if the attribute is not set.
     :param str attribute: The name of the attribute to get the value from. If
-        ``None``, assumes the attribute has the same name as the field.
+        `None`, assumes the attribute has the same name as the field.
     :param str error: Error message stored upon validation failure.
     :param kwargs: The same keyword arguments that :class:`Fixed` receives.
 
@@ -1117,9 +1152,10 @@ class Select(Field):
         self.choices = choices
         return super(Select, self).__init__(default, attribute, error, **kwargs)
 
-    def _validated(self, value, exception_class, *args, **kwargs):
+    def _validated(self, value, exception_class):
         if value not in self.choices:
             raise exception_class(
+                getattr(self, 'error', None) or
                 "{0!r} is not a valid choice for this field.".format(value)
             )
         return value
