@@ -3,13 +3,13 @@
 from __future__ import absolute_import
 
 import datetime
+from decimal import Decimal, ROUND_HALF_EVEN, Context, Inexact
 import functools
 import inspect
 import json
 import time
 import types
 from calendar import timegm
-from decimal import Decimal, Context, Inexact
 from email.utils import formatdate, parsedate
 from pprint import pprint as py_pprint
 
@@ -41,14 +41,12 @@ def is_indexable_but_not_string(obj):
 
 
 def is_collection(obj):
-    """Return True if ``obj`` is a collection type, e.g list, tuple, queryset.
-    """
+    """Return True if ``obj`` is a collection type, e.g list, tuple, queryset."""
     return is_iterable_but_not_string(obj) and not isinstance(obj, dict)
 
 
 def is_instance_or_subclass(val, class_):
-    '''Return True if ``val`` is either a subclass or instance of ``class_``.
-    '''
+    """Return True if ``val`` is either a subclass or instance of ``class_``."""
     try:
         return issubclass(val, class_)
     except TypeError:
@@ -74,6 +72,12 @@ def float_to_decimal(f):
         result = ctx.divide(numerator, denominator)
     return result
 
+ZERO_DECIMAL = Decimal()
+
+def decimal_to_fixed(value, precision):
+    """Convert a `Decimal` to a fixed-precision number as a string."""
+    return text_type(value.quantize(precision, rounding=ROUND_HALF_EVEN))
+
 
 def to_marshallable_type(obj, field_names=None):
     """Helper for converting an object to a dictionary only if it is not
@@ -309,3 +313,16 @@ def get_func_name(func):
     if isinstance(func, functools.partial):
         return func.func.__name__
     return func.__name__
+
+
+def callable_or_raise(obj):
+    """Check that an object is callable, else raise a :exc:`ValueError`.
+    """
+    if not callable(obj):
+        raise ValueError('Object {0!r} is not callable.'.format(obj))
+    return obj
+
+
+def get_func_args(func):
+    """Return a tuple of argument names for a function."""
+    return inspect.getargspec(func).args
